#!/bin/bash -eux
set -o pipefail
# Licensed to Elasticsearch B.V. under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Elasticsearch B.V. licenses this file to you under
# the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# 	http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
# Author: Derek Ditch <dcode@users.noreply.github.com>
# Date:   2022-11-16

# This script depends on the following:
# - bash
# - jq
# - Elastic Security Labs thrunting-tools (https://github.com/elastic/securitylabs-thrunting-tools)
# - iconv
# - GNU awk
# - base64 from GNU Coreutils (the macOS version likely won't work)

# Output a header for the TSV
echo -e "find\treplace\turl1\turl2\tb64code" | tee output.tsv
while read -r line || [[ -n $line ]]; do
    _jq() {
        # This little trick allows you to iterate over a
        # list of base64-encoded JSON documents
        echo "${line}" | base64 --decode | jq -r "${1}"
    }

    # The `escape_string` function sanitizes the string so it
    # can be used in the `gensub` function call as a regular expression
    entry=$(_jq .process.command_line |
        awk -vOFS='|' '
            function escape_string( str ) {
                gsub(/[\\.^$(){}\[\]|*+?]/, "\\\\&", str)
                return str
            }
            {
                match($0, /replace\('\''([^'\'']*)'\'' *, *'\''([^'\'']*)'\''/, arr);
                str=escape_string(arr[1]);
                rep=arr[2];
                clean=gensub(str, rep, "g");

                print clean, str, rep

            }
        ')
    str=$(echo "${entry}" | awk -F\| '{ print $2}')
    repl=$(echo "${entry}" | awk -F\| '{ print $3}')
    # The grep regex here parses out any base64 enclosed by single quotes
    b64code=$(echo "${entry}" | awk -F\| '{ print $1}' | grep -oP ''\''\K[A-Za-z0-9+/]+={0,2}(?='\'';)')

    # We use iconv to translate the decoded powershell to 8-bit text
    code=$(echo "${b64code}" | base64 -d | iconv -f UTF-16 -t UTF-8)

    # Output split by tabs, parse out any HTTP or HTTPS URLs that match forward or backwards
    # If a backwards match, reverse the string
    urls=$(echo "${code}" | awk -vOFS='\t' '
        {
            url1="";
            url2="";
            if ( match($0, /'\''([^'\'']+\/\/:s?ptth)'\''/, arr))
            {
                n=split(arr[1],arr2,"");
                for(i=1;i<=n;i++){s=arr2[i] s};
                url2=s
            };
            if ( match($0, /'\''(https?:\/\/[^'\'']+)'\''/, arr))
            {
                url1=arr[1]
            }

            print url1, url2
        }
    ')

    # Finally output the row, with each field delimited by tabs
    echo -e "${str}\t${repl}\t${urls}\t${b64code}"
done < <(
    eql-query --since 'now-180d/d' --size=1000 --compact --fields process.command_line \
        'intrusion_detection where (process.pe.original_file_name == "PowerShell.EXE" and process.command_line like "*Unicode.GetString*" and process.args like "*replace*")' |
        jq -r '. | @base64' # This encodes the entire JSON object into a single base64-encoded string so we can pass it into the loop
) | tee -a output.tsv
